//
//  CrossControl.swift
//  CrossControl
//
//  Created by kunii on 2018/09/19.
//  Copyright © 2018年 Takahiro Kunii. All rights reserved.
//

import UIKit

///     上下左右のドラッグによりregionプロパティのx,y要素が、それぞれが-1,0,1の値をとるコントローラ
///
///     値に変化があると.valueChangedで知らせる。
///
class CrossControl: UIControl {

    override init(frame: CGRect) {
        super.init(frame:frame)
        setup()
    }
    
    ///  ストーリボード等からの作成に使われる。必須なので実装
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setup()
    }
   
    /// タッチに反応させるためにUIPanGestureRecognizerを設定する
    func setup() {
        let panRecognizer = UIPanGestureRecognizer(target: self, action: #selector(panHandler(_:)))
        //  タッチ情報をUIPanGestureRecognizerに独占させないよう、自身にも送らせるにはfalseにする必要がある
        panRecognizer.cancelsTouchesInView = false
        addGestureRecognizer(panRecognizer)
        //  タッチに反応させるエリアは半透明のグレーにする
        backgroundColor = UIColor(white: 0.5, alpha: 0.5)
    }
    
    /// ドラッグ中の指の存在領域を示す　x,yはそれぞれ-1,0,1のいずれかの値になる
    private var region_ = CGPoint.zero
    
    /// 前回panHandlerが呼び出された時のregion_の値を記録
    private var lastRegion_ = CGPoint.zero

    /// 与えられたドラッグ開始点からの相対位置から　-1,0,1を判定する
    ///
    /// - Parameters:
    ///   - defaultRegion: 無効領域時に使う値
    ///   - translation: ドラッグ開始点からの相対距離
    /// - Returns: 判定値
    private func determineRegion(_ defaultRegion:CGFloat, translation:CGFloat) -> CGFloat {
        /// ドラッグ開始点から、これ以上離れたら0ではない
        let farLimit:CGFloat = 30.0
        
        /// ドラッグ開始点から、これより近い場合は0とする
        let nearLimit:CGFloat = 20.0
        
        if translation < -farLimit {
            return -1
        } else if translation > farLimit {
            return 1
        } else if abs(translation) < nearLimit {
            return 0
        }
        //  停止領域の前後(farLimit - nearLimit)分は無効な領域になっていてdefaultRegionをそのまま返す
        return defaultRegion
    }
    /// ドラッグ対応
    @objc func panHandler(_ panRecognizer:UIPanGestureRecognizer) {
        if panRecognizer.state == .changed {
            
            let translation = panRecognizer.translation(in: panRecognizer.view)
            
            //  ドラッグ中の指の存在領域の決定
            region_.x = determineRegion(region_.x, translation:translation.x)
            region_.y = determineRegion(region_.y, translation:-translation.y)
            
            if !region_.equalTo(lastRegion_) {
                //  領域が変わった
                lastRegion_ = region_
                sendActions(for: .valueChanged)
            }
        }
        if panRecognizer.state == .ended || panRecognizer.state == .cancelled {
            region_ = .zero
            lastRegion_ = .zero
        }
    }
    
    /// 現在の領域（x,yそれぞれの-1, 0, 1）を返す
    var region : CGPoint {
        return region_
    }

}
